///
/// Extended Memory for BML3MK5
///
/// @author Sasaji
/// @date   2016.06.20
///

module MK5EXMEM (
	input wire clk48,
	input wire sw_n,
	output wire [2:0] led,
	//
	inout wire [7:0] data,
	input wire [7:0] addr,
	input wire cs_n,
	input wire re_n,
	input wire we_n,
	input wire ras_n,
	input wire cas_n,
	// for SDRAM
	output wire [12:0] sdram_addr,
	output wire [1:0]  sdram_ba,
	output wire        sdram_cas_n,
	output wire        sdram_cke,
	output wire        sdram_cs_n,
	inout  wire [15:0] sdram_dq,
	output wire [1:0]  sdram_dqm,
	output wire        sdram_ras_n,
	output wire        sdram_we_n,
	output wire        sdram_clk
	// for debug
//	output wire debug_diff_data,
//	output reg [15:0] debug_addr
);
//
wire in_res_n;

// red
assign led[0] = in_res_n;
// green
assign led[1] = (cs_n | cas_n); 
// blue
assign led[2] = 1'b1;

// generate base clock
PLL	uPLL (
	.inclk0 ( clk48 ),
	.c0 ( clk_main ),
	.c1 ( sdram_clk )
);

// power on reset
wire por_n;
POR	uPOR
(
	.clk48	( clk48 ),
	.por_n	( por_n )
);
//
assign in_res_n = (por_n & sw_n);

// access SDRAM (Use QSys)
/*
	input	[23:0]	sdram_controller_0_s1_address;
	input	[1:0]	sdram_controller_0_s1_byteenable_n;
	input		sdram_controller_0_s1_chipselect;
	input	[15:0]	sdram_controller_0_s1_writedata;
	input		sdram_controller_0_s1_read_n;
	input		sdram_controller_0_s1_write_n;
	output	[15:0]	sdram_controller_0_s1_readdata;
	output		sdram_controller_0_s1_readdatavalid;
	output		sdram_controller_0_s1_waitrequest;
*/
reg [23:0] s1_address;
reg [1:0]  s1_byteenable_n;
reg        s1_chipselect;
reg [15:0] s1_writedata;
reg        s1_read_n;
reg        s1_write_n;
wire [15:0] s1_readdata;
wire        s1_readdatavalid;
wire        s1_waitrequest;
	
SDRAM uSDRAM (
	.clk_clk                             ( clk_main ),          //                     clk.clk
	.clk_cs_n                            ( cs_n ),
	.reset_reset_n                       ( in_res_n ),          //                   reset.reset_n
	.sdram_controller_0_wire_addr        ( sdram_addr ),        // sdram_controller_0_wire.addr
	.sdram_controller_0_wire_ba          ( sdram_ba ),          //                        .ba
	.sdram_controller_0_wire_cas_n       ( sdram_cas_n ),       //                        .cas_n
	.sdram_controller_0_wire_cke         ( sdram_cke ),         //                        .cke
	.sdram_controller_0_wire_cs_n        ( sdram_cs_n ),        //                        .cs_n
	.sdram_controller_0_wire_dq          ( sdram_dq ),          //                        .dq
	.sdram_controller_0_wire_dqm         ( sdram_dqm ),         //                        .dqm
	.sdram_controller_0_wire_ras_n       ( sdram_ras_n ),       //                        .ras_n
	.sdram_controller_0_wire_we_n        ( sdram_we_n ),        //                        .we_n
	.sdram_controller_0_s1_address       ( s1_address ),       //   sdram_controller_0_s1.address
	.sdram_controller_0_s1_byteenable_n  ( s1_byteenable_n ),  //                        .byteenable_n
	.sdram_controller_0_s1_chipselect    ( s1_chipselect ),    //                        .chipselect
	.sdram_controller_0_s1_writedata     ( s1_writedata ),     //                        .writedata
	.sdram_controller_0_s1_read_n        ( s1_read_n ),        //                        .read_n
	.sdram_controller_0_s1_write_n       ( s1_write_n ),       //                        .write_n
	.sdram_controller_0_s1_readdata      ( s1_readdata ),      //                        .readdata
	.sdram_controller_0_s1_readdatavalid ( s1_readdatavalid ), //                        .readdatavalid
	.sdram_controller_0_s1_waitrequest   ( s1_waitrequest )    //                        .waitrequest
);

/*
reg in_ras_n;
reg [1:0] in_ras_cnt;
always @(posedge clk_main, negedge in_res_n) begin
	if (~in_res_n) begin
		in_ras_n <= 1'b1;
		in_ras_cnt <= 2'b00;
	end
	else if (ras_n) begin
		if (in_ras_cnt[1]) begin
			in_ras_n <= 1'b1;
		end
		in_ras_cnt <= 2'b10;
	end
	else begin
		if (in_ras_cnt[0]) begin
			in_ras_n <= 1'b0;
		end
		in_ras_cnt <= 2'b01;
	end
end
*/

//
always @(negedge ras_n) begin
	// latch address
	s1_address[15] <= addr[7];
	s1_address[7:1] <= addr[6:0];
end

//
always @(negedge cas_n) begin
	// latch address
	s1_address[14:8] <= addr[7:1];
	s1_address[0] <= addr[0];
end

//
reg [7:0] read_data;
reg [3:0] s1_wait_cnt;
always @(posedge clk_main, negedge in_res_n) begin
	if (~in_res_n) begin
		s1_address[23:16] <= 8'h00; 
		s1_chipselect <= 1'b0;
		s1_read_n <= 1'b1;
		s1_write_n <= 1'b1;
		s1_byteenable_n[1:0] <= 2'b11;

		s1_wait_cnt <= 0;
	end
	else if (cas_n) begin
		s1_chipselect <= 1'b0;
		s1_read_n <= 1'b1;
		s1_write_n <= 1'b1;
		s1_byteenable_n[1:0] <= 2'b11;

		s1_wait_cnt <= 0;
	end
	else begin
		// write
		if (~we_n) begin
			if (s1_wait_cnt == 15) begin
				s1_chipselect <= 1'b0;
				s1_read_n <= 1'b1;
				s1_write_n <= 1'b1;
				s1_byteenable_n[1:0] <= 2'b11;
			end
			else if (s1_wait_cnt == 0) begin
				// write start

				// latch data for write
				s1_writedata <= { 8'h00, data[7:0] };
			
				// sdram chip select
				if (~s1_waitrequest) begin
					s1_wait_cnt <= 1;
				end
			end
			else if (s1_wait_cnt < 15) begin
				// latch data for write
				s1_writedata <= { 8'h00, data[7:0] };

				s1_byteenable_n[1:0] <= 2'b00;
				s1_write_n <= 1'b0;
				s1_chipselect <= 1'b1;

				s1_wait_cnt <= s1_wait_cnt + 1;
			end
		end
		else begin
			// read
			if (s1_wait_cnt == 15) begin
				s1_chipselect <= 1'b0;
				s1_read_n <= 1'b1;
				s1_write_n <= 1'b1;
				s1_byteenable_n[1:0] <= 2'b11;
			end
			else if (s1_wait_cnt == 0) begin
				// read start

				if (~s1_waitrequest) begin
					s1_wait_cnt <= 1;
				end
			end
			else if (s1_wait_cnt < 2) begin
				// sdram chip select
				s1_byteenable_n[1:0] <= 2'b00;
				s1_read_n <= 1'b0;
				s1_chipselect <= 1'b1;

				s1_wait_cnt <= s1_wait_cnt + 1;
			end
			else begin
				if (s1_readdatavalid) begin
					s1_wait_cnt <= 15;
				end
			end
		end
	end
end
//
always @(negedge clk_main) begin
	// data read
	if (~s1_read_n & s1_readdatavalid) begin
		read_data[7:0] <= s1_readdata[7:0];
	end
end

// for debug
//assign debug_diff_data = (read_data[7:0] != s1_writedata[7:0]);

/*
reg [1:0] debug_addr_cas;
always @(posedge clk48, negedge in_res_n) begin
	if (~in_res_n) begin
		debug_addr <= 16'h0000;
		debug_addr_cas <= 0;
	end
	else if (ras_n) begin
		debug_addr[15] <= addr[7];
		debug_addr[7:1] <= addr[6:0];
		debug_addr_cas <= 0;
	end
	else if (debug_addr_cas < 3) begin
		debug_addr_cas <= debug_addr_cas + 1;
	end
	else begin
		debug_addr[14:8] <= addr[7:1];
		debug_addr[0] <= addr[0];
	end

end
*/

//
assign data = (~cs_n & we_n) ? read_data : 8'bzzzzzzzz;

endmodule
