`timescale 1ns/1ps
`define TB_CYCLE 20.833
// 1008KHz
`define E_CYCLE 496.032
// 1008*16KHz
`define C16M_CYCLE 62.004

module MK5EXMEM_tb();

wire in_res_n;
assign in_res_n = uMK5EXMEM.in_res_n;

reg clk48;
initial clk48 = 1'b0;
always #(`TB_CYCLE / 2) clk48 = ~clk48;

reg e1m;
reg ras_n;
reg cas_n;
reg ras_dn;
reg [3:0] c16m_cnt;
initial begin
	e1m = 1'b1;
	ras_n = 1'b1;
	cas_n = 1'b1;
	ras_dn = 1'b1;
	c16m_cnt = 4'h0;
end
always #(`C16M_CYCLE) begin
	c16m_cnt = c16m_cnt + 4'h1;
	if (c16m_cnt >= 4'h8) e1m = 1'b0;
	else e1m = 1'b1;
	if (in_res_n) begin
		if (c16m_cnt >= 4'h3 && c16m_cnt <= 4'h6) ras_n = 1'b0;
		else if (c16m_cnt >= 4'h9 || c16m_cnt <= 4'h0) ras_n = 1'b0;
		else ras_n = 1'b1;
		if (c16m_cnt >= 4'h4 && c16m_cnt <= 4'h7) cas_n = 1'b0;
		else cas_n = 1'b1;
	end
end
always @(ras_n) begin
	#10 ras_dn = ras_n;
end

wire clk_main;
assign clk_main = uMK5EXMEM.clk_main;

reg [31:0] cycles;
initial cycles = 32'h0;
always @(posedge clk_main, negedge in_res_n) begin
	if (~in_res_n) cycles = 32'h0;
	else cycles = cycles + 32'h1;
end

wire [2:0] led;
reg sw_n;

wire [7:0] data;
reg [15:0] addr;
reg re_n;
reg we_n;
wire cs_n;
assign cs_n = ~e1m;

reg [7:0] wdata;

reg phase;
reg rw_phase;

reg [15:0] sdram_wdata;

initial
begin
	sw_n = 1'b1;

	addr = 16'h0000;
	we_n = 1'b1;
	re_n = 1'b0;
	ras_n = 1'b1;
	cas_n = 1'b1;

	wdata = 8'h00;

	phase = 0;
	rw_phase = 0;

	sdram_wdata = 16'h0000;

	uMK5EXMEM.uPOR.por_n_reg = 1'b0;
	uMK5EXMEM.uPOR.por_count = 16'h0000;
end

always @(posedge clk_main) begin

	if (phase > 0) begin
		if (~e1m) begin
			we_n = 1'b1;
			phase = 0;
		end
	end
	else begin
		if (cycles >= 13000) begin
			if (e1m) begin
				addr = addr + 16'h10;
				we_n = rw_phase;
				wdata = ~addr[7:0];
				sdram_wdata = cycles[15:0];
				phase = 1;
				if (addr >= 16'h7ff0) rw_phase = ~rw_phase;
				else if (addr >= 16'h3ff0) rw_phase = 1'b1;
			end
		end
	end
end

wire [7:0] addr8;
assign addr8 = ras_dn ? addr[7:0] : addr[15:8];

wire [12:0] sdram_addr;
wire [1:0]  sdram_ba;
wire        sdram_cas_n;
wire        sdram_cke;
wire        sdram_cs_n;

wire [15:0] sdram_dq;

wire [1:0]  sdram_dqm;
wire        sdram_ras_n;
wire        sdram_we_n;
wire        sdram_clk;

MK5EXMEM uMK5EXMEM (
	.clk48	( clk48 ),
	.sw_n	( sw_n ),
	.led	( led ),
	//
	.data	( data ),
	.addr	( addr8 ),
	.cs_n	( cs_n ),
	.re_n	( re_n ),
	.we_n	( we_n ),
	.ras_n	( ras_n ),
	.cas_n	( cas_n ),
	// for SDRAM
	.sdram_addr		( sdram_addr ),
	.sdram_ba		( sdram_ba ),
	.sdram_cas_n	( sdram_cas_n ),
	.sdram_cke		( sdram_cke ),
	.sdram_cs_n		( sdram_cs_n ),
	.sdram_dq		( sdram_dq ),
	.sdram_dqm		( sdram_dqm ),
	.sdram_ras_n	( sdram_ras_n ),
	.sdram_we_n		( sdram_we_n ),
	.sdram_clk		( sdram_clk )
);
//
assign data = (e1m & ~we_n) ? wdata : 8'bzzzzzzzz;
assign sdram_dq = (~sdram_cs_n & sdram_we_n & ~sdram_cas_n) ? sdram_wdata : 16'bzzzzzzzzzzzzzzzz;

endmodule

